@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
@@ -27,6 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <map>
 #include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include <boost/program_options.hpp>
@@ -35,9 +36,17 @@ namespace po = boost::program_options;
 #include <osmium/io/file.hpp>
 #include <osmium/io/writer_options.hpp>
 #include <osmium/osm/entity_bits.hpp>
-#include <osmium/util/minmax.hpp>
+#include <osmium/util/file.hpp>
 #include <osmium/util/verbose_output.hpp>
 
+const char* get_osmium_version() noexcept;
+const char* get_osmium_long_version() noexcept;
+const char* get_libosmium_version() noexcept;
+
+namespace osmium { namespace io {
+    class Header;
+}}
+
 /**
  * Virtual base class for commands that can be called from the command line.
  */
@@ -45,9 +54,14 @@ class Command {
 
     osmium::osm_entity_bits::type m_osm_entity_bits {osmium::osm_entity_bits::all};
 
+    enum class display_progress_type {
+        never  = 0,
+        on_tty = 1,
+        always = 2
+    } m_display_progress = display_progress_type::on_tty;
+
 protected:
 
-    bool m_debug {false};
     osmium::util::VerboseOutput m_vout {false};
 
 public:
@@ -89,6 +103,10 @@ public:
 
     po::options_description add_common_options();
     void setup_common(const boost::program_options::variables_map& vm, const po::options_description& desc);
+    void setup_progress(const boost::program_options::variables_map& vm);
+    void setup_object_type_nrwc(const boost::program_options::variables_map& vm);
+    void setup_object_type_nrw(const boost::program_options::variables_map& vm);
+    void show_object_types(osmium::util::VerboseOutput& vout);
     void print_arguments(const std::string& command);
     void show_memory_used();
 
@@ -96,13 +114,25 @@ public:
         return m_osm_entity_bits;
     }
 
+    bool display_progress() const {
+        switch (m_display_progress) {
+            case display_progress_type::on_tty:
+                return osmium::util::isatty(2); // if STDERR is a TTY
+            case display_progress_type::always:
+                return true;
+            default:
+                break;
+        }
+        return false;
+    }
+
 }; // class Command
 
 class with_single_osm_input {
 
 protected:
 
-    std::string m_input_filename = "-"; // default: stdin
+    std::string m_input_filename;
     std::string m_input_format;
     osmium::io::File m_input_file;
 
@@ -130,7 +160,7 @@ protected:
 
 public:
 
-    void setup_input_files(const boost::program_options::variables_map& vm, bool optional = false);
+    void setup_input_files(const boost::program_options::variables_map& vm);
 
     po::options_description add_multiple_inputs_options();
 
@@ -148,7 +178,7 @@ protected:
 
     std::string m_generator;
     std::vector<std::string> m_output_headers;
-    std::string m_output_filename = "-"; // default: stdout
+    std::string m_output_filename;
     std::string m_output_format;
     osmium::io::File m_output_file;
     osmium::io::overwrite m_output_overwrite = osmium::io::overwrite::no;
@@ -157,9 +187,12 @@ protected:
 public:
 
     with_osm_output() :
-        m_generator("osmium/" OSMIUM_VERSION) {
+        m_generator("osmium/") {
+        m_generator.append(get_osmium_version());
     }
 
+    void init_output_file(const po::variables_map& vm);
+    void check_output_file();
     void setup_output_file(const po::variables_map& vm);
 
     po::options_description add_output_options();
@@ -170,13 +203,7 @@ public:
         return m_output_file;
     }
 
-    const std::string& generator() const {
-        return m_generator;
-    }
-
-    const std::vector<std::string>& output_headers() const {
-        return m_output_headers;
-    }
+    void setup_header(osmium::io::Header& header) const;
 
     osmium::io::overwrite output_overwrite() const {
         return m_output_overwrite;
@@ -193,7 +220,7 @@ public:
  */
 class CommandFactory {
 
-    typedef std::function<Command*()> create_command_type;
+    using create_command_type = std::function<Command*()>;
 
     struct command_info {
         std::string description; // description of command for help
@@ -229,4 +256,6 @@ public:
 
 }; // class CommandFactory
 
+void register_commands();
+
 #endif // CMD_HPP
