@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,20 +16,35 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
 #include <iostream>
 #include <iterator>
+#include <string>
+#include <utility>
+#include <vector>
 
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/any_output.hpp>
+#include <osmium/index/map.hpp>
+#include <osmium/io/file.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/writer.hpp>
+#include <osmium/memory/buffer.hpp>
+#include <osmium/osm/item_type.hpp>
+#include <osmium/osm/location.hpp>
+#include <osmium/osm/node.hpp>
+#include <osmium/osm/types.hpp>
+#include <osmium/util/progress_bar.hpp>
+#include <osmium/util/verbose_output.hpp>
+#include <osmium/visitor.hpp>
 
 #include "command_add_locations_to_ways.hpp"
 #include "exception.hpp"
+#include "util.hpp"
 
 bool CommandAddLocationsToWays::setup(const std::vector<std::string>& arguments) {
     const auto& map_factory = osmium::index::MapFactory<osmium::unsigned_object_id_type, osmium::Location>::instance();
@@ -40,6 +55,7 @@ bool CommandAddLocationsToWays::setup(const std::vector<std::string>& arguments)
     ("index-type,i", po::value<std::string>()->default_value(default_index_type), "Index type to use")
     ("show-index-types,I", "Show available index types")
     ("keep-untagged-nodes,n", "Keep untagged nodes")
+    ("ignore-missing-nodes", "Ignore missing nodes")
     ;
 
     po::options_description opts_common{add_common_options()};
@@ -74,11 +90,12 @@ bool CommandAddLocationsToWays::setup(const std::vector<std::string>& arguments)
     if (vm.count("index-type")) {
         m_index_type_name = vm["index-type"].as<std::string>();
         if (!map_factory.has_map_type(m_index_type_name)) {
-            throw argument_error(std::string{"Unknown index type '"} + m_index_type_name + "'. Use --show-index-types or -I to get a list.");
+            throw argument_error{std::string{"Unknown index type '"} + m_index_type_name + "'. Use --show-index-types or -I to get a list."};
         }
     }
 
     setup_common(vm, desc);
+    setup_progress(vm);
     setup_input_files(vm);
     setup_output_file(vm);
 
@@ -86,6 +103,10 @@ bool CommandAddLocationsToWays::setup(const std::vector<std::string>& arguments)
         m_keep_untagged_nodes = true;
     }
 
+    if (vm.count("ignore-missing-nodes")) {
+        m_ignore_missing_nodes = true;
+    }
+
     return true;
 }
 
@@ -95,19 +116,13 @@ void CommandAddLocationsToWays::show_arguments() {
 
     m_vout << "  other options:\n";
     m_vout << "    index type: " << m_index_type_name << "\n";
-    m_vout << "    keep untagged nodes: " << (m_keep_untagged_nodes ? "yes" : "no") << "\n";
+    m_vout << "    keep untagged nodes: " << yes_no(m_keep_untagged_nodes);
     m_vout << "\n";
 }
 
-void CommandAddLocationsToWays::setup_header(osmium::io::Header& header) const {
-    header.set("generator", m_generator);
-    for (const auto& h : m_output_headers) {
-        header.set(h);
-    }
-}
-
-void CommandAddLocationsToWays::copy_data(osmium::io::Reader& reader, osmium::io::Writer& writer, location_handler_type& location_handler) {
+void CommandAddLocationsToWays::copy_data(osmium::ProgressBar& progress_bar, osmium::io::Reader& reader, osmium::io::Writer& writer, location_handler_type& location_handler) {
     while (osmium::memory::Buffer buffer = reader.read()) {
+        progress_bar.update(reader.offset());
         osmium::apply(buffer, location_handler);
 
         if (m_keep_untagged_nodes) {
@@ -126,34 +141,43 @@ bool CommandAddLocationsToWays::run() {
     const auto& map_factory = osmium::index::MapFactory<osmium::unsigned_object_id_type, osmium::Location>::instance();
     auto location_index = map_factory.create_map(m_index_type_name);
     location_handler_type location_handler(*location_index);
-    location_handler.ignore_errors(); // XXX
+
+    if (m_ignore_missing_nodes) {
+        location_handler.ignore_errors();
+    }
 
     m_output_file.set("locations_on_ways");
 
     if (m_input_files.size() == 1) { // single input file
         m_vout << "Copying input file '" << m_input_files[0].filename() << "'\n";
-        osmium::io::Reader reader(m_input_files[0]);
-        osmium::io::Header header = reader.header();
+        osmium::io::Reader reader{m_input_files[0]};
+        osmium::io::Header header{reader.header()};
         setup_header(header);
         osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
 
-        copy_data(reader, writer, location_handler);
+        osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
+        copy_data(progress_bar, reader, writer, location_handler);
+        progress_bar.done();
 
         writer.close();
         reader.close();
     } else { // multiple input files
         osmium::io::Header header;
         setup_header(header);
-        osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
+        osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
 
+        osmium::ProgressBar progress_bar{file_size_sum(m_input_files), display_progress()};
         for (const auto& input_file : m_input_files) {
+            progress_bar.remove();
             m_vout << "Copying input file '" << input_file.filename() << "'\n";
             osmium::io::Reader reader(input_file);
 
-            copy_data(reader, writer, location_handler);
+            copy_data(progress_bar, reader, writer, location_handler);
 
+            progress_bar.file_done(reader.file_size());
             reader.close();
         }
+        progress_bar.done();
         writer.close();
     }
 
@@ -163,11 +187,3 @@ bool CommandAddLocationsToWays::run() {
     return true;
 }
 
-namespace {
-
-    const bool register_add_locations_to_ways_command = CommandFactory::add("add-locations-to-ways", "Add node locations to ways", []() {
-        return new CommandAddLocationsToWays();
-    });
-
-}
-
