@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,26 +16,43 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include <algorithm>
+#include <string>
+#include <vector>
+#include <utility>
+
 #include <boost/function_output_iterator.hpp>
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/any_output.hpp>
+#include <osmium/io/file.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/input_iterator.hpp>
 #include <osmium/io/output_iterator.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/writer.hpp>
+#include <osmium/memory/buffer.hpp>
 #include <osmium/object_pointer_collection.hpp>
+#include <osmium/osm/entity_bits.hpp>
+#include <osmium/osm/object.hpp>
 #include <osmium/osm/object_comparisons.hpp>
+#include <osmium/osm/types.hpp>
+#include <osmium/util/verbose_output.hpp>
+#include <osmium/visitor.hpp>
 
 #include "command_apply_changes.hpp"
+#include "exception.hpp"
+#include "util.hpp"
 
 bool CommandApplyChanges::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_cmd{"COMMAND OPTIONS"};
     opts_cmd.add_options()
-    ("simplify,s", "Simplify change")
-    ("remove-deleted,r", "Remove deleted objects from output")
+    ("simplify,s",       "Simplify change (deprecated)")
+    ("remove-deleted,r", "Remove deleted objects from output (deprecated)")
+    ("with-history",     "Apply changes to history file")
     ;
 
     po::options_description opts_common{add_common_options()};
@@ -68,14 +85,29 @@ bool CommandApplyChanges::setup(const std::vector<std::string>& arguments) {
 
     if (vm.count("change-filenames")) {
         m_change_filenames = vm["change-filenames"].as<std::vector<std::string>>();
+    } else {
+        throw argument_error{"Need data file and at least one change file on the command line."};
+    }
+
+    if (vm.count("with-history")) {
+        m_with_history = true;
+        m_output_file.set_has_multiple_object_versions(true);
+    } else {
+        if (m_input_file.has_multiple_object_versions() && m_output_file.has_multiple_object_versions()) {
+            m_with_history = true;
+        } else if (m_input_file.has_multiple_object_versions() != m_output_file.has_multiple_object_versions()) {
+            throw argument_error{"Input and output file must both be OSM data files or both OSM history files (force with --with-history)."};
+        }
     }
 
     if (vm.count("simplify")) {
-        m_simplify_change = true;
+        warning("-s, --simplify option is deprecated. Please see manual page.\n");
+        m_with_history = false;
     }
 
     if (vm.count("remove-deleted")) {
-        m_remove_deleted = true;
+        warning("-r, --remove-deleted option is deprecated. Please see manual page.\n");
+        m_with_history = false;
     }
 
     return true;
@@ -89,41 +121,41 @@ void CommandApplyChanges::show_arguments() {
     }
     m_vout << "  input format: " << m_input_format << "\n";
     show_output_arguments(m_vout);
+    m_vout << "  reading and writing history file: " << yes_no(m_with_history);
 }
 
-/**
- *  Copy the first OSM object with a given Id to the output iterator. Keep
- *  track of the Id of each object to do this.
- *
- *  If kd is set to true, it will copy deleted objects, too. Otherwise those
- *  are suppressed.
- *
- *  We are using this functor class instead of a simple lambda, because the
- *  lambda doesn't build on MSVC.
- */
-class copy_first_with_id {
+namespace {
 
-    osmium::io::OutputIterator<osmium::io::Writer> out;
-    osmium::object_id_type id = 0;
-    bool keep_deleted;
+    /**
+     *  Copy the first OSM object with a given Id to the output. Keep
+     *  track of the Id of each object to do this.
+     *
+     *  We are using this functor class instead of a simple lambda, because the
+     *  lambda doesn't build on MSVC.
+     */
+    class copy_first_with_id {
 
-public:
+        osmium::io::Writer* writer;
+        osmium::object_id_type id = 0;
 
-    copy_first_with_id(osmium::io::OutputIterator<osmium::io::Writer> oi, bool kd) :
-        out(oi),
-        keep_deleted(kd) {
-    }
+    public:
 
-    void operator()(const osmium::OSMObject& obj) {
-        if (obj.id() != id) {
-            if (keep_deleted || obj.visible()) {
-                *out = obj;
-            }
-            id = obj.id();
+        explicit copy_first_with_id(osmium::io::Writer& w) :
+            writer(&w) {
         }
-    }
 
-}; // class copy_first_with_id
+        void operator()(const osmium::OSMObject& obj) {
+            if (obj.id() != id) {
+                if (obj.visible()) {
+                    (*writer)(obj);
+                }
+                id = obj.id();
+            }
+        }
+
+    }; // class copy_first_with_id
+
+} // anonymous namespace
 
 bool CommandApplyChanges::run() {
     std::vector<osmium::memory::Buffer> changes;
@@ -132,7 +164,7 @@ bool CommandApplyChanges::run() {
     m_vout << "Reading change file contents...\n";
 
     for (const std::string& change_file_name : m_change_filenames) {
-        osmium::io::Reader reader(change_file_name, osmium::osm_entity_bits::object);
+        osmium::io::Reader reader{change_file_name, osmium::osm_entity_bits::object};
         while (osmium::memory::Buffer buffer = reader.read()) {
             osmium::apply(buffer, objects);
             changes.push_back(std::move(buffer));
@@ -141,26 +173,38 @@ bool CommandApplyChanges::run() {
     }
 
     m_vout << "Opening input file...\n";
-    osmium::io::Reader reader(m_input_file, osmium::osm_entity_bits::object);
+    osmium::io::Reader reader{m_input_file, osmium::osm_entity_bits::object};
 
-    osmium::io::Header header = reader.header();
-    header.set("generator", m_generator);
+    osmium::io::Header header{reader.header()};
+    setup_header(header);
 
     m_vout << "Opening output file...\n";
-    osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
-    auto out = osmium::io::make_output_iterator(writer);
+    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
 
     auto input = osmium::io::make_input_iterator_range<osmium::OSMObject>(reader);
 
-    if (m_simplify_change) {
-        // If the --simplify option was given we sort with the
-        // largest version of each object first and then only
-        // copy this last version of any object to the output_buffer.
+    if (m_with_history) {
+        // For history files this is a straightforward sort of the change
+        // files followed by a merge with the input file.
+        m_vout << "Sorting change data...\n";
+        objects.sort(osmium::object_order_type_id_version());
+
+        auto out = osmium::io::make_output_iterator(writer);
+        m_vout << "Applying changes and writing them to output...\n";
+        std::set_union(objects.begin(),
+                       objects.end(),
+                       input.begin(),
+                       input.end(),
+                       out);
+    } else {
+        // For normal data files we sort with the largest version of each
+        // object first and then only copy this last version of any object
+        // to the output.
         m_vout << "Sorting change data...\n";
         objects.sort(osmium::object_order_type_id_reverse_version());
 
         auto output_it = boost::make_function_output_iterator(
-                            copy_first_with_id(out, !m_remove_deleted)
+                            copy_first_with_id(writer)
         );
 
         m_vout << "Applying changes and writing them to output...\n";
@@ -170,19 +214,6 @@ bool CommandApplyChanges::run() {
                        input.end(),
                        output_it,
                        osmium::object_order_type_id_reverse_version());
-    } else {
-        // If the --simplify option was not given, this
-        // is a straightforward sort of the change files
-        // and then a merge with the input file.
-        m_vout << "Sorting change data...\n";
-        objects.sort(osmium::object_order_type_id_version());
-
-        m_vout << "Applying changes and writing them to output...\n";
-        std::set_union(objects.begin(),
-                       objects.end(),
-                       input.begin(),
-                       input.end(),
-                       out);
     }
 
     writer.close();
@@ -194,11 +225,3 @@ bool CommandApplyChanges::run() {
     return true;
 }
 
-namespace {
-
-    const bool register_apply_changes_command = CommandFactory::add("apply-changes", "Apply OSM change files to OSM data file", []() {
-        return new CommandApplyChanges();
-    });
-
-}
-
