@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,21 +19,20 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
 #include <string>
 #include <vector>
 
-#include "cmd.hpp"
+#include "cmd.hpp" // IWYU pragma: export
 
 class CommandApplyChanges : public Command, public with_single_osm_input, public with_osm_output {
 
     std::vector<std::string> m_change_filenames;
 
-    bool m_simplify_change = false;
-    bool m_remove_deleted = false;
+    bool m_with_history = false;
 
 public:
 
@@ -50,7 +49,7 @@ public:
     }
 
     const char* synopsis() const noexcept override final {
-        return "osmium apply-changes [OPTIONS] OSM-DATA-FILE OSM-CHANGE-FILE...";
+        return "osmium apply-changes [OPTIONS] OSM-FILE OSM-CHANGE-FILE...";
     }
 
 }; // class CommandApplyChanges
