@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,20 +16,26 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <iostream>
-#include <iterator>
+#include <algorithm>
+#include <string>
+#include <vector>
 
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/any_output.hpp>
+#include <osmium/io/file.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/writer.hpp>
+#include <osmium/memory/buffer.hpp>
+#include <osmium/util/progress_bar.hpp>
+#include <osmium/util/verbose_output.hpp>
 
 #include "command_cat.hpp"
-#include "exception.hpp"
+#include "util.hpp"
 
 bool CommandCat::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_cmd{"COMMAND OPTIONS"};
@@ -60,6 +66,8 @@ bool CommandCat::setup(const std::vector<std::string>& arguments) {
     po::notify(vm);
 
     setup_common(vm, desc);
+    setup_progress(vm);
+    setup_object_type_nrwc(vm);
     setup_input_files(vm);
     setup_output_file(vm);
 
@@ -71,56 +79,45 @@ void CommandCat::show_arguments() {
     show_output_arguments(m_vout);
 
     m_vout << "  other options:\n";
-    m_vout << "    object types:";
-    if (osm_entity_bits() & osmium::osm_entity_bits::node) {
-        m_vout << " node";
-    }
-    if (osm_entity_bits() & osmium::osm_entity_bits::way) {
-        m_vout << " way";
-    }
-    if (osm_entity_bits() & osmium::osm_entity_bits::relation) {
-        m_vout << " relation";
-    }
-    if (osm_entity_bits() & osmium::osm_entity_bits::changeset) {
-        m_vout << " changeset";
-    }
-    m_vout << "\n";
-}
-
-void CommandCat::setup_header(osmium::io::Header& header) const {
-    header.set("generator", m_generator);
-    for (const auto& h : m_output_headers) {
-        header.set(h);
-    }
+    show_object_types(m_vout);
 }
 
 bool CommandCat::run() {
     if (m_input_files.size() == 1) { // single input file
         m_vout << "Copying input file '" << m_input_files[0].filename() << "'\n";
-        osmium::io::Reader reader(m_input_files[0], osm_entity_bits());
-        osmium::io::Header header = reader.header();
+        osmium::io::Reader reader{m_input_files[0], osm_entity_bits()};
+        osmium::io::Header header{reader.header()};
         setup_header(header);
         osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
 
+        osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
         while (osmium::memory::Buffer buffer = reader.read()) {
+            progress_bar.update(reader.offset());
             writer(std::move(buffer));
         }
+        progress_bar.done();
+
         writer.close();
         reader.close();
     } else { // multiple input files
         osmium::io::Header header;
         setup_header(header);
-        osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
+        osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
 
+        osmium::ProgressBar progress_bar{file_size_sum(m_input_files), display_progress()};
         for (const auto& input_file : m_input_files) {
+            progress_bar.remove();
             m_vout << "Copying input file '" << input_file.filename() << "'\n";
-            osmium::io::Reader reader(input_file, osm_entity_bits());
+            osmium::io::Reader reader{input_file, osm_entity_bits()};
             while (osmium::memory::Buffer buffer = reader.read()) {
+                progress_bar.update(reader.offset());
                 writer(std::move(buffer));
             }
+            progress_bar.file_done(reader.file_size());
             reader.close();
         }
         writer.close();
+        progress_bar.done();
     }
 
     show_memory_used();
@@ -129,11 +126,3 @@ bool CommandCat::run() {
     return true;
 }
 
-namespace {
-
-    const bool register_cat_command = CommandFactory::add("cat", "Concatenate OSM files and convert to different formats", []() {
-        return new CommandCat();
-    });
-
-}
-
