@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,22 +19,21 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
 #include <string>
 #include <vector>
 
-#include <osmium/io/header.hpp>
-#include <osmium/osm/entity_bits.hpp>
+#include "cmd.hpp" // IWYU pragma: export
 
-#include "cmd.hpp"
+namespace osmium { namespace io {
+    class Header;
+}}
 
 class CommandCat : public Command, public with_multiple_osm_inputs, public with_osm_output {
 
-    void setup_header(osmium::io::Header& header) const;
-
 public:
 
     CommandCat() = default;
