@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,18 +16,28 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include <algorithm>
 #include <stdexcept>
+#include <string>
+#include <vector>
 
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/any_output.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/writer.hpp>
 #include <osmium/io/input_iterator.hpp>
 #include <osmium/io/output_iterator.hpp>
+#include <osmium/osm/changeset.hpp>
+#include <osmium/osm/entity_bits.hpp>
+#include <osmium/osm/timestamp.hpp>
+#include <osmium/osm/types.hpp>
+#include <osmium/util/progress_bar.hpp>
+#include <osmium/util/verbose_output.hpp>
 
 #include "command_changeset_filter.hpp"
 #include "exception.hpp"
@@ -70,6 +80,7 @@ bool CommandChangesetFilter::setup(const std::vector<std::string>& arguments) {
     po::notify(vm);
 
     setup_common(vm, desc);
+    setup_progress(vm);
     setup_input_file(vm);
     setup_output_file(vm);
 
@@ -109,8 +120,8 @@ bool CommandChangesetFilter::setup(const std::vector<std::string>& arguments) {
         auto ts = vm["after"].as<std::string>();
         try {
             m_after = osmium::Timestamp(ts);
-        } catch (std::invalid_argument&) {
-            throw argument_error("Wrong format for --after/-a timestamp (use YYYY-MM-DDThh:mm:ssZ).");
+        } catch (const std::invalid_argument&) {
+            throw argument_error{"Wrong format for --after/-a timestamp (use YYYY-MM-DDThh:mm:ssZ)."};
         }
     }
 
@@ -118,25 +129,25 @@ bool CommandChangesetFilter::setup(const std::vector<std::string>& arguments) {
         auto ts = vm["before"].as<std::string>();
         try {
             m_before = osmium::Timestamp(ts);
-        } catch (std::invalid_argument&) {
-            throw argument_error("Wrong format for --before/-b timestamp (use YYYY-MM-DDThh:mm:ssZ).");
+        } catch (const std::invalid_argument&) {
+            throw argument_error{"Wrong format for --before/-b timestamp (use YYYY-MM-DDThh:mm:ssZ)."};
         }
     }
 
     if (m_with_discussion && m_without_discussion) {
-        throw argument_error("You can not use --with-discussion/-d and --without-discussion/-D together.");
+        throw argument_error{"You can not use --with-discussion/-d and --without-discussion/-D together."};
     }
 
     if (m_with_changes && m_without_changes) {
-        throw argument_error("You can not use --with-changes/-c and --without-changes/-C together.");
+        throw argument_error{"You can not use --with-changes/-c and --without-changes/-C together."};
     }
 
     if (m_open && m_closed) {
-        throw argument_error("You can not use --open and --closed together.");
+        throw argument_error{"You can not use --open and --closed together."};
     }
 
     if (m_after > m_before) {
-        throw argument_error("Timestamp 'after' is after 'before'.");
+        throw argument_error{"Timestamp 'after' is after 'before'."};
     }
 
     return true;
@@ -189,21 +200,28 @@ bool changeset_before(const osmium::Changeset& changeset, osmium::Timestamp time
 
 bool CommandChangesetFilter::run() {
     m_vout << "Opening input file...\n";
-    osmium::io::Reader reader(m_input_file, osmium::osm_entity_bits::changeset);
+    osmium::io::Reader reader{m_input_file, osmium::osm_entity_bits::changeset};
 
     auto input = osmium::io::make_input_iterator_range<osmium::Changeset>(reader);
 
-    osmium::io::Header header = reader.header();
-    header.set("generator", m_generator);
+    osmium::io::Header header{reader.header()};
+    setup_header(header);
 
     m_vout << "Opening output file...\n";
-    osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
+    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
     auto out = osmium::io::make_output_iterator(writer);
 
     m_vout << "Filtering data...\n";
 
+    osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
+
+    int count = 0;
     std::copy_if(input.begin(), input.end(), out,
-        [this](const osmium::Changeset& changeset) {
+        [this, &count, &progress_bar, &reader](const osmium::Changeset& changeset) {
+            if (++count > 10000) {
+                progress_bar.update(reader.offset());
+                count = 0;
+            }
             return (!m_with_discussion    || changeset.num_comments() > 0) &&
                    (!m_without_discussion || changeset.num_comments() == 0) &&
                    (!m_with_changes       || changeset.num_changes() > 0) &&
@@ -217,6 +235,8 @@ bool CommandChangesetFilter::run() {
 
     });
 
+    progress_bar.done();
+
     writer.close();
     reader.close();
 
@@ -226,11 +246,3 @@ bool CommandChangesetFilter::run() {
     return true;
 }
 
-namespace {
-
-    const bool register_changeset_filter_command = CommandFactory::add("changeset-filter", "Filter OSM changesets by different criteria", []() {
-        return new CommandChangesetFilter();
-    });
-
-}
-
