@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,17 +16,22 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
 #include <algorithm>
+#include <cerrno>
+#include <cstdint>
 #include <iostream>
 #include <iterator>
+#include <memory>
 #include <sstream>
+#include <string>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <system_error>
+#include <utility>
 #include <vector>
 
 #ifndef _MSC_VER
@@ -42,15 +47,21 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <boost/crc.hpp>
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
 #include <osmium/handler.hpp>
+#include <osmium/io/file.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/osm.hpp>
+#include <osmium/osm/box.hpp>
 #include <osmium/osm/crc.hpp>
-#include <osmium/osm/entity_bits.hpp>
 #include <osmium/util/minmax.hpp>
+#include <osmium/util/progress_bar.hpp>
+#include <osmium/util/verbose_output.hpp>
 #include <osmium/visitor.hpp>
 
 #include "command_fileinfo.hpp"
 #include "exception.hpp"
+#include "util.hpp"
 
 /*************************************************************************/
 
@@ -80,6 +91,15 @@ struct InfoHandler : public osmium::handler::Handler {
     osmium::object_id_type last_id = 0;
 
     void changeset(const osmium::Changeset& changeset) {
+        if (last_type == osmium::item_type::changeset) {
+            if (last_id > changeset.id()) {
+                ordered = false;
+            }
+        } else {
+            last_type = osmium::item_type::changeset;
+        }
+
+        last_id = changeset.id();
         crc32.update(changeset);
         ++changesets;
 
@@ -97,7 +117,7 @@ struct InfoHandler : public osmium::handler::Handler {
             if (last_id > object.id()) {
                 ordered = false;
             }
-        } else if (last_type > object.type()) {
+        } else if (last_type != osmium::item_type::changeset && last_type > object.type()) {
             ordered = false;
         }
 
@@ -138,7 +158,7 @@ off_t filesize(const std::string& filename) {
 
     struct stat s;
     if (::stat(filename.c_str(), &s) == -1) {
-        throw std::system_error(errno, std::system_category(), "stat failed");
+        throw std::system_error{errno, std::system_category(), std::string{"Could not get file size of file '"} + filename + "'"};
     }
 
     return s.st_size;
@@ -183,7 +203,7 @@ public:
         for (const auto& box : header.boxes()) {
             std::cout << "    " << box << "\n";
         }
-        std::cout << "  With history: " << (header.has_multiple_object_versions() ? "yes" : "no") << "\n";
+        std::cout << "  With history: " << yes_no(header.has_multiple_object_versions());
 
         std::cout << "  Options:\n";
         for (const auto& option : header) {
@@ -201,11 +221,11 @@ public:
             std::cout << "    Last: " << info_handler.last_timestamp() << "\n";
         }
 
-        std::cout << "  Objects ordered (by type and id): " << (info_handler.ordered ? "yes\n" : "no\n");
+        std::cout << "  Objects ordered (by type and id): " << yes_no(info_handler.ordered);
 
         std::cout << "  Multiple versions of same object: ";
         if (info_handler.ordered) {
-            std::cout << (info_handler.multiple_versions ? "yes\n" : "no\n");
+            std::cout << yes_no(info_handler.multiple_versions);
             if (info_handler.multiple_versions != header.has_multiple_object_versions()) {
                 std::cout << "    WARNING! This is different from the setting in the header.\n";
             }
@@ -231,7 +251,7 @@ public:
 
 class JSONOutput : public Output {
 
-    typedef rapidjson::PrettyWriter<rapidjson::StringBuffer> writer_type;
+    using writer_type = rapidjson::PrettyWriter<rapidjson::StringBuffer>;
 
     rapidjson::StringBuffer m_stream;
     writer_type m_writer;
@@ -371,13 +391,9 @@ class SimpleOutput : public Output {
 
     std::string m_get_value;
 
-    bool output_variable(const std::string& name) {
-        return m_get_value == name;
-    }
-
 public:
 
-    SimpleOutput(const std::string& get_value) :
+    explicit SimpleOutput(const std::string& get_value) :
         m_get_value(get_value) {
     }
 
@@ -402,11 +418,11 @@ public:
 
     void header(const osmium::io::Header& header) override final {
         if (m_get_value == "header.with_history") {
-            std::cout << (header.has_multiple_object_versions() ? "yes" : "no") << "\n";
+            std::cout << yes_no(header.has_multiple_object_versions());
         }
 
         for (const auto& option : header) {
-            std::string value_name = "header.option.";
+            std::string value_name{"header.option."};
             value_name.append(option.first);
             if (m_get_value == value_name) {
                 std::cout << option.second << "\n";
@@ -514,6 +530,7 @@ bool CommandFileinfo::setup(const std::vector<std::string>& arguments) {
     po::notify(vm);
 
     setup_common(vm, desc);
+    setup_progress(vm);
 
     if (vm.count("extended")) {
         m_extended = true;
@@ -562,15 +579,15 @@ bool CommandFileinfo::setup(const std::vector<std::string>& arguments) {
         m_get_value = vm["get"].as<std::string>();
         const auto& f = std::find(known_values.cbegin(), known_values.cend(), m_get_value);
         if (f == known_values.cend()) {
-            throw argument_error(std::string("Unknown value for --get/-g option '") + m_get_value + "'. Use --show-variables/-G to see list of known values.");
+            throw argument_error{std::string{"Unknown value for --get/-g option '"} + m_get_value + "'. Use --show-variables/-G to see list of known values."};
         }
         if (m_get_value.substr(0, 5) == "data." && ! m_extended) {
-            throw argument_error("You need to set --extended/-e for any 'data.*' variables to be available.");
+            throw argument_error{"You need to set --extended/-e for any 'data.*' variables to be available."};
         }
     }
 
     if (vm.count("get") && vm.count("json")) {
-        throw argument_error("You can not use --get/-g and --json/-j together.");
+        throw argument_error{"You can not use --get/-g and --json/-j together."};
     }
 
     return true;
@@ -595,13 +612,18 @@ bool CommandFileinfo::run() {
 
     output->file(m_input_filename, m_input_file);
 
-    osmium::io::Reader reader(m_input_file, m_extended ? osmium::osm_entity_bits::all : osmium::osm_entity_bits::nothing);
-    osmium::io::Header header = reader.header();
+    osmium::io::Reader reader{m_input_file, m_extended ? osmium::osm_entity_bits::all : osmium::osm_entity_bits::nothing};
+    osmium::io::Header header{reader.header()};
     output->header(header);
 
     if (m_extended) {
         InfoHandler info_handler;
-        osmium::apply(reader, info_handler);
+        osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
+        while (osmium::memory::Buffer buffer = reader.read()) {
+            progress_bar.update(reader.offset());
+            osmium::apply(buffer, info_handler);
+        }
+        progress_bar.done();
         output->data(header, info_handler);
     }
 
@@ -613,11 +635,3 @@ bool CommandFileinfo::run() {
     return true;
 }
 
-namespace {
-
-    const bool register_fileinfo_command = CommandFactory::add("fileinfo", "Show information about OSM file", []() {
-        return new CommandFileinfo();
-    });
-
-}
-
