@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,38 +16,41 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include <cstddef>
 #include <fstream>
-#include <iterator>
-#include <numeric>
+#include <iostream>
+#include <map>
+#include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/any_output.hpp>
-#include <osmium/osm/entity_bits.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/writer.hpp>
+#include <osmium/memory/buffer.hpp>
+#include <osmium/osm.hpp>
 #include <osmium/osm/types_from_string.hpp>
+#include <osmium/util/progress_bar.hpp>
 #include <osmium/util/string.hpp>
+#include <osmium/util/verbose_output.hpp>
 
 #include "command_getid.hpp"
 #include "exception.hpp"
-
-std::set<osmium::object_id_type>& CommandGetId::ids(osmium::item_type type) noexcept {
-    return m_ids[osmium::item_type_to_nwr_index(type)];
-}
-
-const std::set<osmium::object_id_type>& CommandGetId::ids(osmium::item_type type) const noexcept {
-    return m_ids[osmium::item_type_to_nwr_index(type)];
-}
+#include "util.hpp"
 
 void CommandGetId::parse_and_add_id(const std::string& s) {
     auto p = osmium::string_to_object_id(s.c_str(), osmium::osm_entity_bits::nwr, m_default_item_type);
-    ids(p.first).insert(p.second);
+    if (p.second < 0) {
+        throw std::runtime_error("osmium-getid does not work with negative IDs");
+    }
+    m_ids(p.first).set(p.second);
 }
 
 void CommandGetId::read_id_file(std::istream& stream) {
@@ -64,23 +67,23 @@ void CommandGetId::read_id_file(std::istream& stream) {
 }
 
 bool CommandGetId::no_ids() const {
-    return ids(osmium::item_type::node).empty() &&
-           ids(osmium::item_type::way).empty() &&
-           ids(osmium::item_type::relation).empty();
+    return m_ids(osmium::item_type::node).empty() &&
+           m_ids(osmium::item_type::way).empty() &&
+           m_ids(osmium::item_type::relation).empty();
 }
 
-size_t CommandGetId::count_ids() const {
-    return ids(osmium::item_type::node).size() +
-           ids(osmium::item_type::way).size() +
-           ids(osmium::item_type::relation).size();
+std::size_t CommandGetId::count_ids() const {
+    return m_ids(osmium::item_type::node).size() +
+           m_ids(osmium::item_type::way).size() +
+           m_ids(osmium::item_type::relation).size();
 }
 
 bool CommandGetId::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_cmd{"COMMAND OPTIONS"};
     opts_cmd.add_options()
     ("default-type", po::value<std::string>()->default_value("node"), "Default item type")
-    ("id-file,i", po::value<std::string>(), "Read OSM IDs from text file")
-    ("id-osm-file,I", po::value<std::string>(), "Read OSM IDs from OSM file")
+    ("id-file,i", po::value<std::vector<std::string>>(), "Read OSM IDs from text file")
+    ("id-osm-file,I", po::value<std::vector<std::string>>(), "Read OSM IDs from OSM file")
     ("history,H", "Make it work with history files")
     ("add-referenced,r", "Recursively add referenced objects")
     ("verbose-ids", "Print all requested and missing IDs")
@@ -111,12 +114,13 @@ bool CommandGetId::setup(const std::vector<std::string>& arguments) {
     po::notify(vm);
 
     setup_common(vm, desc);
+    setup_progress(vm);
     setup_input_file(vm);
     setup_output_file(vm);
 
     if (vm.count("add-referenced")) {
         if (m_input_filename == "-") {
-            throw argument_error("Can not read OSM input from STDIN when -r/--add-referenced option is used.");
+            throw argument_error{"Can not read OSM input from STDIN when --add-referenced/-r option is used."};
         }
         m_add_referenced_objects = true;
     }
@@ -135,7 +139,7 @@ bool CommandGetId::setup(const std::vector<std::string>& arguments) {
         } else if (t == "r" || t == "relation") {
             m_default_item_type = osmium::item_type::relation;
         } else {
-            throw argument_error(std::string("Unknown default type '") + t + "' (Allowed are 'node', 'way', and 'relation').");
+            throw argument_error{std::string{"Unknown default type '"} + t + "' (Allowed are 'node', 'way', and 'relation')."};
         }
     }
 
@@ -145,24 +149,26 @@ bool CommandGetId::setup(const std::vector<std::string>& arguments) {
     }
 
     if (vm.count("id-file")) {
-        std::string filename = vm["id-file"].as<std::string>();
-
-        if (filename == "-") {
-            if (m_input_filename == "-") {
-                throw argument_error("Can not read OSM input and IDs both from STDIN.");
+        for (const std::string& filename : vm["id-file"].as<std::vector<std::string>>()) {
+            if (filename == "-") {
+                if (m_input_filename == "-") {
+                    throw argument_error{"Can not read OSM input and IDs both from STDIN."};
+                }
+                read_id_file(std::cin);
+            } else {
+                std::ifstream id_file{filename};
+                if (!id_file.is_open()) {
+                    throw argument_error{"Could not open file '" + filename + "'"};
+                }
+                read_id_file(id_file);
             }
-            read_id_file(std::cin);
-        } else {
-            std::ifstream id_file{filename};
-            if (!id_file.is_open()) {
-                throw argument_error("Could not open file '" + filename + "'");
-            }
-            read_id_file(id_file);
         }
     }
 
     if (vm.count("id-osm-file")) {
-        read_id_osm_file(vm["id-osm-file"].as<std::string>());
+        for (const std::string& filename : vm["id-osm-file"].as<std::vector<std::string>>()) {
+            read_id_osm_file(filename);
+        }
     }
 
     if (vm.count("ids")) {
@@ -176,7 +182,7 @@ bool CommandGetId::setup(const std::vector<std::string>& arguments) {
     }
 
     if (no_ids()) {
-        throw argument_error("Please specify IDs to look for on command line or with option -i/--id-file or -I/--id-osm-file.");
+        throw argument_error{"Please specify IDs to look for on command line or with option --id-file/-i or --id-osm-file/-I."};
     }
 
     return true;
@@ -187,43 +193,43 @@ void CommandGetId::show_arguments() {
     show_output_arguments(m_vout);
 
     m_vout << "  other options:\n";
-    m_vout << "    add referenced objects: " << (m_add_referenced_objects ? "yes" : "no") << "\n";
-    m_vout << "    work with history files: " << (m_work_with_history ? "yes" : "no") << "\n";
+    m_vout << "    add referenced objects: " << yes_no(m_add_referenced_objects);
+    m_vout << "    work with history files: " << yes_no(m_work_with_history);
     m_vout << "    default object type: " << osmium::item_type_to_name(m_default_item_type) << "\n";
     if (m_verbose_ids) {
         m_vout << "    looking for these ids:\n";
         m_vout << "      nodes:";
-        for (osmium::object_id_type id : ids(osmium::item_type::node)) {
+        for (osmium::object_id_type id : m_ids(osmium::item_type::node)) {
             m_vout << " " << id;
         }
         m_vout << "\n";
         m_vout << "      ways:";
-        for (osmium::object_id_type id : ids(osmium::item_type::way)) {
+        for (osmium::object_id_type id : m_ids(osmium::item_type::way)) {
             m_vout << " " << id;
         }
         m_vout << "\n";
         m_vout << "      relations:";
-        for (osmium::object_id_type id : ids(osmium::item_type::relation)) {
+        for (osmium::object_id_type id : m_ids(osmium::item_type::relation)) {
             m_vout << " " << id;
         }
         m_vout << "\n";
     } else {
-        m_vout << "    looking for " << ids(osmium::item_type::node).size() << " node ID(s), "
-                                     << ids(osmium::item_type::way).size() << " way ID(s), and "
-                                     << ids(osmium::item_type::relation).size() << " relation ID(s)\n";
+        m_vout << "    looking for " << m_ids(osmium::item_type::node).size() << " node ID(s), "
+                                     << m_ids(osmium::item_type::way).size() << " way ID(s), and "
+                                     << m_ids(osmium::item_type::relation).size() << " relation ID(s)\n";
     }
 }
 
 osmium::osm_entity_bits::type CommandGetId::get_needed_types() const {
     osmium::osm_entity_bits::type types = osmium::osm_entity_bits::nothing;
 
-    if (! ids(osmium::item_type::node).empty()) {
+    if (! m_ids(osmium::item_type::node).empty()) {
         types |= osmium::osm_entity_bits::node;
     }
-    if (! ids(osmium::item_type::way).empty()) {
+    if (! m_ids(osmium::item_type::way).empty()) {
         types |= osmium::osm_entity_bits::way;
     }
-    if (! ids(osmium::item_type::relation).empty()) {
+    if (! m_ids(osmium::item_type::relation).empty()) {
         types |= osmium::osm_entity_bits::relation;
     }
 
@@ -232,17 +238,17 @@ osmium::osm_entity_bits::type CommandGetId::get_needed_types() const {
 
 void CommandGetId::add_nodes(const osmium::Way& way) {
     for (const auto& nr : way.nodes()) {
-        ids(osmium::item_type::node).insert(nr.ref());
+        m_ids(osmium::item_type::node).set(nr.positive_ref());
     }
 }
 
 void CommandGetId::add_members(const osmium::Relation& relation) {
     for (const auto& member : relation.members()) {
-        ids(member.type()).insert(member.ref());
+        m_ids(member.type()).set(member.positive_ref());
     }
 }
 
-static void print_missing_ids(const char* type, std::set<osmium::object_id_type>& set) {
+static void print_missing_ids(const char* type, const osmium::index::IdSetDense<osmium::unsigned_object_id_type>& set) {
     if (set.empty()) {
         return;
     }
@@ -255,10 +261,10 @@ static void print_missing_ids(const char* type, std::set<osmium::object_id_type>
 
 void CommandGetId::read_id_osm_file(const std::string& file_name) {
     m_vout << "Reading OSM ID file...\n";
-    osmium::io::Reader reader(file_name, osmium::osm_entity_bits::object);
+    osmium::io::Reader reader{file_name, osmium::osm_entity_bits::object};
     while (osmium::memory::Buffer buffer = reader.read()) {
         for (const auto& object : buffer.select<osmium::OSMObject>()) {
-            ids(object.type()).insert(object.id());
+            m_ids(object.type()).set(object.positive_id());
             if (object.type() == osmium::item_type::way) {
                 add_nodes(static_cast<const osmium::Way&>(object));
             } else if (object.type() == osmium::item_type::relation) {
@@ -272,8 +278,7 @@ void CommandGetId::read_id_osm_file(const std::string& file_name) {
 void CommandGetId::mark_rel_ids(const std::multimap<osmium::object_id_type, osmium::object_id_type>& rel_in_rel, osmium::object_id_type id) {
     auto range = rel_in_rel.equal_range(id);
     for (auto it = range.first; it != range.second; ++it) {
-        if (ids(osmium::item_type::relation).count(it->second) == 0) {
-            ids(osmium::item_type::relation).insert(it->second);
+        if (m_ids(osmium::item_type::relation).check_and_set(it->second)) {
             mark_rel_ids(rel_in_rel, it->second);
         }
     }
@@ -283,17 +288,17 @@ bool CommandGetId::find_relations_in_relations() {
     m_vout << "  Reading input file to find relations in relations...\n";
     std::multimap<osmium::object_id_type, osmium::object_id_type> rel_in_rel;
 
-    osmium::io::Reader reader(m_input_file, osmium::osm_entity_bits::relation);
+    osmium::io::Reader reader{m_input_file, osmium::osm_entity_bits::relation};
     while (osmium::memory::Buffer buffer = reader.read()) {
         for (const auto& relation : buffer.select<osmium::Relation>()) {
             for (const auto& member : relation.members()) {
                 if (member.type() == osmium::item_type::relation) {
                     rel_in_rel.emplace(relation.id(), member.ref());
-                } else if (ids(osmium::item_type::relation).count(relation.id())) {
+                } else if (m_ids(osmium::item_type::relation).get(relation.positive_id())) {
                     if (member.type() == osmium::item_type::node) {
-                        ids(osmium::item_type::node).insert(member.ref());
+                        m_ids(osmium::item_type::node).set(member.positive_ref());
                     } else if (member.type() == osmium::item_type::way) {
-                        ids(osmium::item_type::way).insert(member.ref());
+                        m_ids(osmium::item_type::way).set(member.positive_ref());
                     }
                 }
             }
@@ -305,7 +310,7 @@ bool CommandGetId::find_relations_in_relations() {
         return false;
     }
 
-    for (const osmium::object_id_type id : ids(osmium::item_type::relation)) {
+    for (const osmium::unsigned_object_id_type id : m_ids(osmium::item_type::relation)) {
         mark_rel_ids(rel_in_rel, id);
     }
 
@@ -315,15 +320,15 @@ bool CommandGetId::find_relations_in_relations() {
 void CommandGetId::find_nodes_and_ways_in_relations() {
     m_vout << "  Reading input file to find nodes/ways in relations...\n";
 
-    osmium::io::Reader reader(m_input_file, osmium::osm_entity_bits::relation);
+    osmium::io::Reader reader{m_input_file, osmium::osm_entity_bits::relation};
     while (osmium::memory::Buffer buffer = reader.read()) {
         for (const auto& relation : buffer.select<osmium::Relation>()) {
-            if (ids(osmium::item_type::relation).count(relation.id())) {
+            if (m_ids(osmium::item_type::relation).get(relation.positive_id())) {
                 for (const auto& member : relation.members()) {
                     if (member.type() == osmium::item_type::node) {
-                        ids(osmium::item_type::node).insert(member.ref());
+                        m_ids(osmium::item_type::node).set(member.positive_ref());
                     } else if (member.type() == osmium::item_type::way) {
-                        ids(osmium::item_type::way).insert(member.ref());
+                        m_ids(osmium::item_type::way).set(member.positive_ref());
                     }
                 }
             }
@@ -335,10 +340,10 @@ void CommandGetId::find_nodes_and_ways_in_relations() {
 void CommandGetId::find_nodes_in_ways() {
     m_vout << "  Reading input file to find nodes in ways...\n";
 
-    osmium::io::Reader reader(m_input_file, osmium::osm_entity_bits::way);
+    osmium::io::Reader reader{m_input_file, osmium::osm_entity_bits::way};
     while (osmium::memory::Buffer buffer = reader.read()) {
         for (const auto& way : buffer.select<osmium::Way>()) {
-            if (ids(osmium::item_type::way).count(way.id())) {
+            if (m_ids(osmium::item_type::way).get(way.positive_id())) {
                 add_nodes(way);
             }
         }
@@ -348,7 +353,7 @@ void CommandGetId::find_nodes_in_ways() {
 
 void CommandGetId::find_referenced_objects() {
     m_vout << "Following references...\n";
-    bool todo = !ids(osmium::item_type::relation).empty();
+    bool todo = !m_ids(osmium::item_type::relation).empty();
     if (todo) {
         todo = find_relations_in_relations();
     }
@@ -357,7 +362,7 @@ void CommandGetId::find_referenced_objects() {
         find_nodes_and_ways_in_relations();
     }
 
-    if (!ids(osmium::item_type::way).empty()) {
+    if (!m_ids(osmium::item_type::way).empty()) {
         find_nodes_in_ways();
     }
     m_vout << "Done following references.\n";
@@ -369,25 +374,28 @@ bool CommandGetId::run() {
     }
 
     m_vout << "Opening input file...\n";
-    osmium::io::Reader reader(m_input_file, get_needed_types());
+    osmium::io::Reader reader{m_input_file, get_needed_types()};
 
     m_vout << "Opening output file...\n";
     osmium::io::Header header = reader.header();
-    header.set("generator", m_generator);
-    osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
+    setup_header(header);
+
+    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
 
     m_vout << "Copying from source to output file...\n";
+    osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
     while (osmium::memory::Buffer buffer = reader.read()) {
+        progress_bar.update(reader.offset());
         for (const auto& object : buffer.select<osmium::OSMObject>()) {
-            auto& index = ids(object.type());
-            if (index.count(object.id())) {
+            if (m_ids(object.type()).get(object.positive_id())) {
                 if (!m_work_with_history) {
-                    index.erase(object.id());
+                    m_ids(object.type()).unset(object.positive_id());
                 }
                 writer(object);
             }
         }
     }
+    progress_bar.done();
 
     m_vout << "Closing output file...\n";
     writer.close();
@@ -401,9 +409,9 @@ bool CommandGetId::run() {
         } else {
             m_vout << "Did not find " << count_ids() << " object(s).\n";
             if (m_verbose_ids) {
-                print_missing_ids("node",     ids(osmium::item_type::node));
-                print_missing_ids("way",      ids(osmium::item_type::way));
-                print_missing_ids("relation", ids(osmium::item_type::relation));
+                print_missing_ids("node",     m_ids(osmium::item_type::node));
+                print_missing_ids("way",      m_ids(osmium::item_type::way));
+                print_missing_ids("relation", m_ids(osmium::item_type::relation));
             }
         }
     }
@@ -415,11 +423,3 @@ bool CommandGetId::run() {
     return m_work_with_history || no_ids();
 }
 
-namespace {
-
-    const bool register_get_id_command = CommandFactory::add("getid", "Get objects with given ID from OSM file", []() {
-        return new CommandGetId();
-    });
-
-}
-
