@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
@@ -29,11 +29,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <vector>
 
 #include <osmium/fwd.hpp>
+#include <osmium/index/id_set.hpp>
 #include <osmium/osm/entity_bits.hpp>
 #include <osmium/osm/item_type.hpp>
 #include <osmium/osm/types.hpp>
 
-#include "cmd.hpp"
+#include "cmd.hpp" // IWYU pragma: export
 
 class CommandGetId : public Command, public with_single_osm_input, public with_osm_output {
 
@@ -43,10 +44,7 @@ class CommandGetId : public Command, public with_single_osm_input, public with_o
     bool m_work_with_history = false;
     bool m_verbose_ids = false;
 
-    std::set<osmium::object_id_type> m_ids[3];
-
-    std::set<osmium::object_id_type>& ids(osmium::item_type type) noexcept;
-    const std::set<osmium::object_id_type>& ids(osmium::item_type type) const noexcept;
+    osmium::index::NWRIdSet<osmium::index::IdSetDense> m_ids;
 
     void parse_and_add_id(const std::string& s);
 
@@ -55,7 +53,7 @@ class CommandGetId : public Command, public with_single_osm_input, public with_o
 
     osmium::osm_entity_bits::type get_needed_types() const;
     bool no_ids() const;
-    size_t count_ids() const;
+    std::size_t count_ids() const;
 
     void find_referenced_objects();
 
