@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,17 +16,14 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <algorithm>
-#include <cstring>
 #include <iostream>
 #include <iomanip>
-#include <map>
-#include <numeric>
 #include <string>
+#include <vector>
 
 #ifndef _MSC_VER
 # include <unistd.h>
@@ -39,17 +36,17 @@ bool CommandHelp::setup(const std::vector<std::string>& arguments) {
     return true;
 }
 
-void show_help(const std::string& topic, const std::string& info) {
+static void show_help(const std::string& topic, const std::string& info) {
 #ifndef _MSC_VER
     // show man page on non-Windows systems
-    std::string manpage("osmium-");
+    std::string manpage{"osmium-"};
     manpage += topic;
     ::execlp("man", "man", manpage.c_str(), nullptr);
     // if exec fails, fall thru
 #endif
     // show info string and link on Windows systems
     std::cout << info << "\n";
-    std::cout << "You'll find more documentation at http://osmcode.org/osmium/\n";
+    std::cout << "You'll find more documentation at http://osmcode.org/osmium-tool/\n";
 }
 
 bool CommandHelp::run() {
@@ -78,7 +75,7 @@ bool CommandHelp::run() {
 
     std::string description = CommandFactory::get_description(m_topic);
     if (!description.empty()) {
-        show_help(m_topic, std::string("osmium ") + m_topic + ": " + description);
+        show_help(m_topic, std::string{"osmium "} + m_topic + ": " + description);
         return true;
     }
 
@@ -91,11 +88,3 @@ bool CommandHelp::run() {
     return false;
 }
 
-namespace {
-
-    const bool register_help_command = CommandFactory::add("help", "Show osmium help", []() {
-        return new CommandHelp();
-    });
-
-}
-
