@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,22 +16,33 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <iostream>
+#include <algorithm>
+#include <string>
+#include <vector>
 
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/any_output.hpp>
+#include <osmium/io/header.hpp>
 #include <osmium/io/output_iterator.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/writer.hpp>
+#include <osmium/memory/buffer.hpp>
 #include <osmium/object_pointer_collection.hpp>
+#include <osmium/osm/entity_bits.hpp>
 #include <osmium/osm/object_comparisons.hpp>
+#include <osmium/util/verbose_output.hpp>
+#include <osmium/visitor.hpp>
 
 #include "command_merge_changes.hpp"
 
+namespace osmium { namespace io {
+    class File;
+}}
+
 bool CommandMergeChanges::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_cmd{"COMMAND OPTIONS"};
     opts_cmd.add_options()
@@ -79,8 +90,9 @@ void CommandMergeChanges::show_arguments() {
 bool CommandMergeChanges::run() {
     m_vout << "Opening output file...\n";
     osmium::io::Header header;
-    header.set("generator", m_generator);
-    osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
+    setup_header(header);
+
+    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
     auto out = osmium::io::make_output_iterator(writer);
 
     // this will contain all the buffers with the input data
@@ -92,7 +104,7 @@ bool CommandMergeChanges::run() {
     // to each object to objects collection.
     m_vout << "Reading change file contents...\n";
     for (osmium::io::File& change_file : m_input_files) {
-        osmium::io::Reader reader(change_file, osmium::osm_entity_bits::object);
+        osmium::io::Reader reader{change_file, osmium::osm_entity_bits::object};
         while (osmium::memory::Buffer buffer = reader.read()) {
             osmium::apply(buffer, objects);
             changes.push_back(std::move(buffer));
@@ -128,11 +140,3 @@ bool CommandMergeChanges::run() {
     return true;
 }
 
-namespace {
-
-    const bool register_merge_changes_command = CommandFactory::add("merge-changes", "Merge several OSM change files into one", []() {
-        return new CommandMergeChanges();
-    });
-
-}
-
