@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,18 +16,27 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <boost/function_output_iterator.hpp>
+#include <algorithm>
+#include <string>
+#include <vector>
+
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/any_output.hpp>
+#include <osmium/io/header.hpp>
 #include <osmium/io/output_iterator.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/writer.hpp>
+#include <osmium/memory/buffer.hpp>
 #include <osmium/object_pointer_collection.hpp>
+#include <osmium/osm/box.hpp>
+#include <osmium/osm/entity_bits.hpp>
 #include <osmium/osm/object_comparisons.hpp>
+#include <osmium/util/verbose_output.hpp>
+#include <osmium/visitor.hpp>
 
 #include "command_sort.hpp"
 
@@ -78,8 +87,8 @@ bool CommandSort::run() {
 
     m_vout << "Reading contents of input files...\n";
     for (const std::string& file_name : m_filenames) {
-        osmium::io::Reader reader(file_name, osmium::osm_entity_bits::object);
-        auto header = reader.header();
+        osmium::io::Reader reader{file_name, osmium::osm_entity_bits::object};
+        osmium::io::Header header{reader.header()};
         bounding_box.extend(header.joined_boxes());
         while (osmium::memory::Buffer buffer = reader.read()) {
             osmium::apply(buffer, objects);
@@ -90,11 +99,12 @@ bool CommandSort::run() {
 
     m_vout << "Opening output file...\n";
     osmium::io::Header header;
+    setup_header(header);
     if (bounding_box) {
         header.add_box(bounding_box);
     }
-    header.set("generator", m_generator);
-    osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
+
+    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
 
     m_vout << "Sorting data...\n";
     objects.sort(osmium::object_order_type_id_version());
@@ -112,11 +122,3 @@ bool CommandSort::run() {
     return true;
 }
 
-namespace {
-
-    const bool register_sort_command = CommandFactory::add("sort", "Sort OSM data files", []() {
-        return new CommandSort();
-    });
-
-}
-
