@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,22 +16,31 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include <algorithm>
+#include <ctime>
 #include <stdexcept>
 
 #include <boost/program_options.hpp>
 
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/any_output.hpp>
+#include <osmium/diff_iterator.hpp>
+#include <osmium/io/file.hpp>
+#include <osmium/io/header.hpp>
 #include <osmium/io/input_iterator.hpp>
 #include <osmium/io/output_iterator.hpp>
-#include <osmium/diff_iterator.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/writer.hpp>
+#include <osmium/osm/diff_object.hpp>
+#include <osmium/osm/entity_bits.hpp>
+#include <osmium/osm/object.hpp>
+#include <osmium/util/verbose_output.hpp>
 
 #include "command_time_filter.hpp"
 #include "exception.hpp"
+#include "util.hpp"
 
 bool CommandTimeFilter::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_common{add_common_options()};
@@ -64,41 +73,41 @@ bool CommandTimeFilter::setup(const std::vector<std::string>& arguments) {
     setup_input_file(vm);
     setup_output_file(vm);
 
-    m_from = osmium::Timestamp(time(0));
+    m_from = osmium::Timestamp(std::time(0));
     m_to = m_from;
 
     if (vm.count("time-from")) {
-        auto ts = vm["time-from"].as<std::string>();
+        const auto ts = vm["time-from"].as<std::string>();
         try {
-            m_from = osmium::Timestamp(ts);
-        } catch (std::invalid_argument&) {
-            throw argument_error("Wrong format for (first) timestamp (use YYYY-MM-DDThh:mm:ssZ).");
+            m_from = osmium::Timestamp{ts};
+        } catch (const std::invalid_argument&) {
+            throw argument_error{"Wrong format for (first) timestamp (use YYYY-MM-DDThh:mm:ssZ)."};
         }
         m_to = m_from;
     }
 
     if (vm.count("time-to")) {
-        auto ts = vm["time-to"].as<std::string>();
+        const auto ts = vm["time-to"].as<std::string>();
         try {
-            m_to = osmium::Timestamp(ts);
-        } catch (std::invalid_argument&) {
-            throw argument_error("Wrong format for second timestamp (use YYYY-MM-DDThh:mm:ssZ).");
+            m_to = osmium::Timestamp{ts};
+        } catch (const std::invalid_argument&) {
+            throw argument_error{"Wrong format for second timestamp (use YYYY-MM-DDThh:mm:ssZ)."};
         }
     }
 
     if (m_from > m_to) {
-        throw argument_error("Second timestamp is before first one.");
+        throw argument_error{"Second timestamp is before first one."};
     }
 
     if (m_from == m_to) { // point in time
         if (m_output_file.has_multiple_object_versions()) {
-            std::cerr << "Warning! You are writing to a file marked as having multiple object versions,\n";
-            std::cerr << "but there will be only a single version of each object.\n";
+            warning("You are writing to a file marked as having multiple object versions,\n"
+                    "but there will be only a single version of each object.\n");
         }
     } else { // time range
         if (!m_output_file.has_multiple_object_versions()) {
-            std::cerr << "Warning! You are writing to a file marked as having a single object version,\n";
-            std::cerr << "but there might be multiple versions of each object.\n";
+            warning("You are writing to a file marked as having a single object version,\n"
+                    "but there might be multiple versions of each object.\n");
         }
     }
 
@@ -114,12 +123,13 @@ void CommandTimeFilter::show_arguments() {
 
 bool CommandTimeFilter::run() {
     m_vout << "Opening input file...\n";
-    osmium::io::Reader reader(m_input_file, osmium::osm_entity_bits::object);
+    osmium::io::Reader reader{m_input_file, osmium::osm_entity_bits::object};
 
     m_vout << "Opening output file...\n";
-    osmium::io::Header header = reader.header();
-    header.set("generator", m_generator);
-    osmium::io::Writer writer(m_output_file, header, m_output_overwrite, m_fsync);
+    osmium::io::Header header{reader.header()};
+    setup_header(header);
+
+    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
 
     m_vout << "Filter data while copying it from input to output...\n";
     auto input = osmium::io::make_input_iterator_range<osmium::OSMObject>(reader);
@@ -157,11 +167,3 @@ bool CommandTimeFilter::run() {
     return true;
 }
 
-namespace {
-
-    const bool register_time_filter_command = CommandFactory::add("time-filter", "Filter OSM data from a point in time or a time span out of a history file", []() {
-        return new CommandTimeFilter();
-    });
-
-}
-
