@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,12 +16,27 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include <cstddef>
+#include <string>
+#include <vector>
+
+#include <boost/program_options.hpp>
+
+#include <osmium/io/any_input.hpp> // IWYU pragma: keep
+#include <osmium/io/any_output.hpp> // IWYU pragma: keep
+#include <osmium/io/file.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/writer_options.hpp>
+#include <osmium/util/file.hpp>
+#include <osmium/util/verbose_output.hpp>
+
 #include "cmd.hpp"
 #include "exception.hpp"
+#include "util.hpp"
 
 void with_single_osm_input::setup_input_file(const boost::program_options::variables_map& vm) {
     if (vm.count("input-filename")) {
@@ -32,11 +47,19 @@ void with_single_osm_input::setup_input_file(const boost::program_options::varia
         m_input_format = vm["input-format"].as<std::string>();
     }
 
-    if ((m_input_filename == "-" || m_input_filename == "") && m_input_format.empty()) {
-        throw argument_error("When reading from STDIN you need to use the --input-format/-F option to declare the file format.");
+    if (m_input_format.empty()) {
+        if (m_input_filename == "-") {
+            throw argument_error{"When reading from STDIN you need to use the --input-format/-F option\n"
+                                 "to specify the file format."};
+        }
+
+        if (m_input_filename == "") {
+            throw argument_error{"Missing input file. Use '-' to read from STDIN and add the --input-format/-F\n"
+                                 "option to specify the file format or specify the input file name."};
+        }
     }
 
-    m_input_file = osmium::io::File(m_input_filename, m_input_format);
+    m_input_file = osmium::io::File{m_input_filename, m_input_format};
 }
 
 po::options_description with_single_osm_input::add_single_input_options() {
@@ -55,31 +78,34 @@ void with_single_osm_input::show_single_input_arguments(osmium::util::VerboseOut
     vout << "    file format: " << m_input_format << "\n";
 }
 
-void with_multiple_osm_inputs::setup_input_files(const boost::program_options::variables_map& vm, bool optional) {
+void with_multiple_osm_inputs::setup_input_files(const boost::program_options::variables_map& vm) {
     if (vm.count("input-filenames")) {
         m_input_filenames = vm["input-filenames"].as<std::vector<std::string>>();
-    } else if (!optional) {
+    } else {
         m_input_filenames.push_back("-"); // default is stdin
     }
 
+    bool uses_stdin = false;
+    for (auto& filename : m_input_filenames) {
+        if (filename == "-") {
+            if (uses_stdin) {
+                throw argument_error{"Can read at most one file from STDIN."};
+            }
+            uses_stdin = true;
+        }
+    }
+
     if (vm.count("input-format")) {
         m_input_format = vm["input-format"].as<std::string>();
     }
 
-    if (m_input_format.empty()) {
-        bool uses_stdin = false;
-        for (auto& filename : m_input_filenames) {
-            if (filename.empty() || filename == "-") {
-                uses_stdin = true;
-            }
-        }
-        if (uses_stdin) {
-            throw argument_error("When reading from STDIN you need to use the --input-format/-F option to declare the file format.");
-        }
+    if (uses_stdin && m_input_format.empty()) {
+        throw argument_error{"When reading from STDIN you need to use the --input-format/-F option\n"
+                             "to specify the file format. Or are you missing a file name argument?"};
     }
 
     for (const std::string& input_filename : m_input_filenames) {
-        osmium::io::File input_file(input_filename, m_input_format);
+        osmium::io::File input_file{input_filename, m_input_format};
         m_input_files.push_back(input_file);
     }
 }
@@ -104,7 +130,7 @@ void with_multiple_osm_inputs::show_multiple_inputs_arguments(osmium::util::Verb
     vout << "    file format: " << m_input_format << "\n";
 }
 
-void with_osm_output::setup_output_file(const po::variables_map& vm) {
+void with_osm_output::init_output_file(const po::variables_map& vm) {
     if (vm.count("generator")) {
         m_generator = vm["generator"].as<std::string>();
     }
@@ -128,15 +154,29 @@ void with_osm_output::setup_output_file(const po::variables_map& vm) {
     if (vm.count("fsync")) {
         m_fsync = osmium::io::fsync::yes;
     }
+}
 
-    if ((m_output_filename == "-" || m_output_filename == "") && m_output_format.empty()) {
-        throw argument_error("When writing to STDOUT you need to use the --output-format/-f option to declare the file format.");
+void with_osm_output::check_output_file() {
+    if (m_output_format.empty()) {
+        if (m_output_filename == "-") {
+            throw argument_error{"When writing to STDOUT you need to use the --output-format/-f\n"
+                                 "option to specify the file format."};
+        }
+        if (m_output_filename == "") {
+            throw argument_error{"Missing output file. Set the output file with --output/-o and/or\n"
+                                 "add the --input-format/-F option to specify the file format."};
+        }
     }
 
-    m_output_file = osmium::io::File(m_output_filename, m_output_format);
+    m_output_file = osmium::io::File{m_output_filename, m_output_format};
     m_output_file.check();
 }
 
+void with_osm_output::setup_output_file(const po::variables_map& vm) {
+    init_output_file(vm);
+    check_output_file();
+}
+
 po::options_description with_osm_output::add_output_options() {
     po::options_description options("OUTPUT OPTIONS");
 
@@ -157,8 +197,8 @@ void with_osm_output::show_output_arguments(osmium::util::VerboseOutput& vout) {
     vout << "    file name: " << m_output_filename << "\n";
     vout << "    file format: " << m_output_format << "\n";
     vout << "    generator: " << m_generator << "\n";
-    vout << "    overwrite: " << (m_output_overwrite == osmium::io::overwrite::allow ? "yes" : "no") << "\n";
-    vout << "    fsync: " << (m_fsync == osmium::io::fsync::yes ? "yes" : "no") << "\n";
+    vout << "    overwrite: " << yes_no(m_output_overwrite == osmium::io::overwrite::allow);
+    vout << "    fsync: " << yes_no(m_fsync == osmium::io::fsync::yes);
     if (!m_output_headers.empty()) {
         vout << "    output header:\n";
         for (const auto& h : m_output_headers) {
@@ -167,3 +207,10 @@ void with_osm_output::show_output_arguments(osmium::util::VerboseOutput& vout) {
     }
 }
 
+void with_osm_output::setup_header(osmium::io::Header& header) const {
+    header.set("generator", m_generator);
+    for (const auto& h : m_output_headers) {
+        header.set(h);
+    }
+}
+
