@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium
+http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,11 +16,15 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include <cerrno>
+#include <exception>
 #include <iostream>
+#include <memory>
+#include <new>
 #include <string>
 #include <vector>
 
@@ -29,7 +33,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # include <io.h>
 #endif
 
-#include <osmium/version.hpp>
+#include <boost/program_options/errors.hpp>
+
+#include <osmium/handler/check_order.hpp>
 
 #include "cmd.hpp"
 
@@ -44,6 +50,8 @@ int main(int argc, char *argv[]) {
     _setmode(1, _O_BINARY);
 #endif
 
+    register_commands();
+
     std::string command = argv[0];
 
     // remove path from command
@@ -78,10 +86,10 @@ int main(int argc, char *argv[]) {
     }
 
     if (command == "version") {
-        std::cout << "osmium version " OSMIUM_VERSION "\n"
-                  << "libosmium version " LIBOSMIUM_VERSION_STRING "\n"
-                  << "Copyright (C) 2013-2016  Jochen Topf <jochen@topf.org>\n"
-                  << "License: GNU GENERAL PUBLIC LICENSE Version 3 <http://gnu.org/licenses/gpl.html>.\n"
+        std::cout << get_osmium_long_version() << '\n'
+                  << get_libosmium_version() << '\n'
+                  << "Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>\n"
+                  << "License: GNU GENERAL PUBLIC LICENSE Version 3 <https://gnu.org/licenses/gpl.html>.\n"
                   << "This is free software: you are free to change and redistribute it.\n"
                   << "There is NO WARRANTY, to the extent permitted by law.\n";
 
@@ -99,11 +107,11 @@ int main(int argc, char *argv[]) {
         if (!cmd->setup(arguments)) {
             return return_code::okay;
         }
-    } catch (boost::program_options::error& e) {
-        std::cerr << "Error parsing command line: " << e.what() << std::endl;
+    } catch (const boost::program_options::error& e) {
+        std::cerr << "Error parsing command line: " << e.what() << '\n';
         return return_code::fatal;
-    } catch (std::exception& e) {
-        std::cerr << e.what() << std::endl;
+    } catch (const std::exception& e) {
+        std::cerr << e.what() << '\n';
         return return_code::fatal;
     }
 
@@ -113,10 +121,20 @@ int main(int argc, char *argv[]) {
         if (cmd->run()) {
             return return_code::okay;
         }
-    } catch (std::bad_alloc& e) {
+    } catch (const std::bad_alloc&) {
         std::cerr << "Out of memory. Read the MEMORY USAGE section of the osmium(1) manpage.\n";
-    } catch (std::exception& e) {
-        std::cerr << e.what() << "\n";
+    } catch (const osmium::out_of_order_error& e) {
+        std::cerr << e.what() << '\n';
+        std::cerr << "This command expects the input file to be ordered: First nodes in order of ID,\n"
+                  << "then ways in order of ID, then relations in order of ID.\n";
+    } catch (const std::system_error& e) {
+        std::cerr << e.what();
+        if (e.code().value() == EEXIST) {
+            std::cerr << ". Try using --overwrite if you are sure you want to overwrite the file.";
+        }
+        std::cerr << '\n';
+    } catch (const std::exception& e) {
+        std::cerr << e.what() << '\n';
     }
 
     return return_code::error;
